


define(['jquery', 'backbone', 'marionette', 'underscore', 'hbs/handlebars', 'json2xml', 'md5', 'apps/common/lib/spinner','i18next'],
    function($, Backbone, Marionette, _, Handlebars, json2xml, md5, spinner, i18n) {
        var App = new Backbone.Marionette.Application();
        window.App = App;
        //Organize Application into regions corresponding to DOM elements
        //Regions can contain views, Layouts, or subregions nested as necessary


        var MainRegion = Backbone.Marionette.Region.extend({
            el: '#main-region',
            open: function(view) {
                this.$el.hide();
                this.$el.html(view.el);
                this.$el.fadeIn('fast');
            }
        });
        var modalRegion = Backbone.Marionette.Region.extend({
            el: '#modal-region',
            open: function(view) {
                $('#modalbg').fadeIn('fast');
                this.$el.hide();
                this.$el.html(view.el);
                this.$el.fadeIn('fast');

            },
            close: function() {
                $('#modalbg').fadeOut('fast');
                this.$el.html('');
                this.$el.fadeOut(100);

                $('#modalbg').hide();

            }
        });

        var notificationRegion = Backbone.Marionette.Region.extend({
            el: '#notification-region',
            initialize: function() {

            },
            open: function(view) {
                var self = this;


                self.$el.html(view.el);
                $('#notification-region').animate({
                    top: '0px'
                }, 300);

            },
            close: function() {
                var self = this;
                $('#notification-region').animate({
                    top: '-50px'
                }, 130, function() {});
                self.$el.html('');

            }
        });

        var PanningRegion = Marionette.Region.extend({



            slide: function(newView, type) {
                var oposite;
                switch (type) {
                    case 'left':
                        oposite = 'right';
                        break;
                    case 'right':
                        oposite = 'left';
                        break;

                }
                setTimeout(function() {
                    newView.$el.addClass('center');
                    newView.$el.removeClass('left');

                }, 100);
            },
            slideShow: function(newView, type) {

                if (this.running) {
                    return;
                }

                var self = this;

                var view = this.currentView;

                if (!view || view.isClosed) {
                    this.show(newView);
                    var field = document.createElement('input');
                    field.setAttribute('type', 'text');
                    self.currentView.el.appendChild(field);
                    field.focus();
                    field.setAttribute('style', 'display:none;');
                    setTimeout(function() {
                        field.setAttribute('style', 'display:none;');

                    }, 2);

                    return;
                }

                type = 'left';


                this.stopListening(newView, 'render');

                spinner(false);

                if (this.running) {

                    return;
                }
                this.running = true;
                var renderF = function() {

                    self.$el.off('webkitTransitionEnd transitionend msTransitionEnd oTransitionEnd');
                    newView.$el.addClass('sliding');
                    newView.$el.addClass(type);
                    self.$el.append(newView.el);

                    if (device.platform === 'iOS') {
                        cordova.plugins.Keyboard.close();
                    }


                    if (view.oid !== 'map_view') {
                        self.close();
                        view.close();
                    } else {
                        newView.$el.addClass('sliding');
                        self.currentView.$el.detach();
                    }

                    self.$el.on('webkitTransitionEnd transitionend msTransitionEnd oTransitionEnd', function(e) {

                        if ($(e.target).hasClass('sliding')) {
                            window.blur();
                            self.$el.off('webkitTransitionEnd transitionend msTransitionEnd oTransitionEnd');
                            newView.$el.removeClass('center');


                            Marionette.triggerMethod.call(newView, 'show');

                            self.currentView = newView;
                            self.running = false;
                        } else {

                            self.running = false;
                        }
                    });
                };

                if (newView && newView.oid === 'map_view') {


                    self.$el.off('webkitTransitionEnd transitionend msTransitionEnd oTransitionEnd');
                    newView.$el.addClass('sliding');
                    newView.$el.addClass(type);
                    self.$el.append(newView.el);

                    if (newView._firstRender) {

                        newView.render();
                    }

                    //Borro la anterior
                    self.close();
                    view.close();

                    self.$el.on('webkitTransitionEnd transitionend msTransitionEnd oTransitionEnd', function(e) {

                        if ($(e.target).hasClass('sliding')) {

                            // body...
                            self.$el.off('webkitTransitionEnd transitionend msTransitionEnd oTransitionEnd');
                            newView.$el.removeClass('left');
                            newView.$el.removeClass('right');
                            newView.$el.removeClass('center');

                            self.currentView = newView;
                            Marionette.triggerMethod.call(newView, 'show');
                            Marionette.triggerMethod.call(self, 'show', newView);
                            self.running = false;

                        }

                    });
                } else {

                    this.listenTo(newView, 'render', renderF);
                    newView.render();
                }
                this.slide(newView, type);
                if (newView.oid !== 'conversation' && newView.oid !== 'map_view') {


                    setTimeout(function() {
                        window.scrollTo(0, 0);
                    }, 1);
                    newView.oid  = '';
                }

            }

        });
        App.addRegions({
            //mainRegion: MainRegion,
            mainRegion: {
                selector: '#main-region',
                regionType: PanningRegion
            },
            menu: '#site-menu',
            modal: {
                selector: '#modal-region',
                regionType: modalRegion
            },
            notification: {
                selector: '#notification-region',
                regionType: notificationRegion
            }
        });

        App.layout = function() {
            var width = $(document).width();

            if (width <= 767)
                return 'phone';
            if (width > 767 && width <= 979)
                return 'tablet';

            return 'desktop';
        };

        App.commands.setHandler('app:notification', function(nick, msg,data) {



            if(App.mainRegion.currentView.oid === 'conversation' && App.mainRegion.currentView.id_client == data.id_client){

                return;
            }

            console.log(msg);
            if(msg.search('<img')!==-1){
                msg = i18n.t('19_chat.newImage');
            }
            var tpl = '<div class="pict"></div><div class="info"><div class="Nick">' + nick + '</div><div class="msg">{{cleanmessage \''+msg+'\'}}</div><div class="close"></div></div>';
            console.log('tpl',tpl)
            var Nview = Marionette.ItemView.extend({

                template: Handlebars.compile(tpl),
                triggers: {
                    'click .close': 'close',
                    'click .info':'open',
                    'click .pict':'open'

                }



            });
            var view = new Nview({});
            view.on('close', function() {
                App.notification.close();
            });
            view.on('open', function() {
                App.execute('app:chat:conversation',data.sender, data);
                App.notification.close();
            });
            App.notification.show(view);

        });
        App.commands.setHandler('app:close', function() {

            navigator.app.exitApp();
        });


        var checkGoogle = function(num) {
            if (typeof google === 'undefined') {
                if (num > 10) {

                    App.execute('network_manager:force');
                    return;
                }
                setTimeout(function() {
                    checkGoogle(num + 1);
                }, 1000);
            } else {

                Backbone.history.start();

                App.execute('show_login');

            }
        };


        App.addInitializer(function() {
            checkGoogle(0);


        });


        return App;
    });
