


define(['App',
        'underscore',
        'handlebars',
        'moment',
        'i18next',
        'hbs!apps/chat/conversation/templates/conversation',
        'hbs!apps/chat/conversation/templates/item',
        'apps/login/picture/views'

    ],
    function(App, _, Handlebars, moment, i18n, conversation, itemTpl, ImageView) {
        var prof = App.request('entities:profile');
        App.module('chat.conversation.views', function(Module, App, Backbone, Marionette) {

            Module.ImageUpload = ImageView.ImageUpload;



            Module.Item = Marionette.ItemView.extend({
                template: itemTpl,
                className: 'chat-itm'

            });
            Module.List = Marionette.CompositeView.extend({
                triggers: {
                    'click .back': 'back',
                    'click .conv-center': 'quickview',
                    'click .conv-right': 'quickmap'
                },
                events: {
                    'submit form': 'sendmsg',
                    'click .js-add-photo': 'addPhoto',
                    'click .js-picker': 'hidePhoto',
                    'click .js-open-gallery': 'openGallery',
                    'click .js-take-photo': 'takePhoto',
                    'click .js-send-message': 'sendmsg',
                    'focus textarea': 'focusFn',
                    'blur textarea': 'blurFn',
                    'keypress textarea': 'checkTextAreaSize'

                },
                addPhoto: function(e) {
                    e.preventDefault();
                    this.$el.find('.js-picker').toggleClass('active');

                },
                hidePhoto: function() {
                    this.$el.find('.js-picker').removeClass('active');

                },
                printImage: function(data, msg) {
                    var img = '<img class="js-open-image" src="data:image/png;base64,' + data + '" data-src="data:image/png;base64,' + data + '" style="max-width: 120px;height: auto;"/>';

                    var newMsg = _.clone(msg);

                    newMsg.message = img;

                    var toAdd = itemTpl(newMsg);
                    this.$('.container').append('<p id="mess_' + msg.date + '">' + toAdd + '</p>');

                    this.$el.find('div.container .blank').remove();
                    this.$el.find('div.container').append('<div class="blank"></div>');
                    this.scrollEnd();
                    this.model.set('pending', 0);
                    this.model.save();

                },

                initialize: function() {
                    var self = this;
                    this.sendClicked = false;
                    this.model.set('pending', 0);
                    if (device.platform === 'iOS') {
                        self.iosFixInterval = setInterval(function() {
                            self.IOSFix();
                        }, 50);
                    }

                },
                onClose: function() {
                    if (device.platform === 'iOS') {
                        clearInterval(this.iosFixInterval);
                    }
                },

                updateMessageReceived: function(date) {

                    this.$('.msg_' + date).prepend('<div class="chatcheck">');


                },
                newMessage: function(msg) {

                    var toAdd;
                    if (parseInt(msg.sender, 10) === parseInt(prof.get('id_client'), 10)) {
                        msg.mine = true;
                        if (this.$('#mess_' + msg.date).length === 0) {

                            toAdd = itemTpl(msg);
                            this.$('.container.chat_list').append('<p>' + toAdd + '</p>');
                        }
                    } else {

                        toAdd = itemTpl(msg);
                        $('.container.chat_list').append('<p>' + toAdd + '</p>');
                    }
                    if (device.platform === 'iOS') {
                        this.$el.find('div.container .blank').remove();
                        this.$el.find('div.container').append('<div class="blank"></div>');
                    }
                    this.scrollEnd();
                    this.model.set('pending', 0);
                    this.model.save();
                    //if (device.platform === 'iOS') {
                    this.scrollEnd();
                    //}

                },
                template: conversation,
                itemView: Module.Item,
                itemViewContainer: 'div.container',
                blurFn: function() {
                    $('.toolbar').css('pointer-events', 'auto');
                    this.IOSFix();
                    this.$('.write').removeClass('multiline');
                },
                focusFn: function() {
                    $('.toolbar').css('pointer-events', 'none');
                    this.IOSFix();
                    this.checkTextAreaSize();

                },
                IOSFix: function() {
                    if (device.platform === 'iOS') {
                        setTimeout(function() {
                            // body...
                            var top = $(window).scrollTop() + 'px';
                            $('.toolbar').css('top', top);
                        }, 0);
                    }
                },
                openGallery: function(e) {
                    e.preventDefault();
                    this.hidePhoto();
                    this.trigger('openGallery');
                },
                takePhoto: function(e) {
                    e.preventDefault();
                    this.hidePhoto();
                    this.trigger('takePhoto');
                },
                previewImage: function(data) {

                    var self = this;
                    App.noBack = true;
                    //Preload de img...
                    var img = new Image();
                    img.src = 'data:image/png;base64,' + data;
                    var $overlay = $('<div class="fullscreen"></div>');
                    $overlay.html('<div style="max-width: 95%;max-height: 95%; position:relative;">' +
                        '<img class="fullscreen-img" src="' + img.src +
                        '" style="max-width: 100%;max-height: 100%;"><div class="image-buttons"></div>');
                    $overlay.find('.image-buttons').html('<span class="js-close">' + i18n.t('app.cancel') + '</span><span class="js-send right">' + i18n.t('app.send') + '</span></div>');


                    var alreadyShown = $('.fullscreen').length > 0;

                    if (!alreadyShown) {

                        $('body').append($overlay[0]);
                        $overlay.find('.js-close').eq(0).one('click', function() {
                            App.noBack = false;
                            $overlay.remove();

                        });
                        $overlay.find('.js-send').eq(0).one('click', function() {
                            App.noBack = false;
                            $overlay.remove();
                            self.trigger('send_image', data);
                        });

                    }

                },
                closeImage: function(event, $overlay) {
                    if (App.noBack) {

                        if (event) {
                            $overlay = $('.fullscreen');
                        }
                        App.noBack = false;
                        $overlay.remove();

                    }
                },
                openImage: function(e) {
                    App.noBack = true;

                    //Preload...
                    var i = new Image();
                    i.src = $(e.target).attr('data-src');

                    var $overlay = $('<div class="fullscreen"></div>');
                    $overlay.html('<div style="max-width: 95%;max-height: 95%; position:relative;"><img class="fullscreen-img" src="' + i.src + '" style="max-width: 100%;max-height: 100%;"><span style="position: absolute;top: -0.2em;right: -0.2em;background: black;display: inline-block;width: 1em;height: 1em;text-align: center;color: white;line-height: 1;font-size: 2em;border-radius: 50%;">×</span></div>');

                    var alreadyShown = $('.fullscreen').length > 0;




                    if (!alreadyShown) {

                        $('body').append($overlay[0]);
                        $overlay.on('click', function() {
                            App.noBack = false;
                            $overlay.remove();

                        });

                    }

                },
                normal: function() {
                    var self = this;
                    setTimeout(function() {
                        self.$('nav').css('position', 'fixed');
                        self.$('.blank').css('display', 'inherit');
                        self.$('.write').css('position', 'fixed');
                        self.$('.write').css('bottom', '0');

                        self.scrollEnd();


                    }, 10);

                },
                className: 'conv-chat',
                onRender: function() {

                    var self = this;
                    $(document).on('click', '.js-open-image', self.openImage);
                    $(document).on('backbutton', self.closeImage);

                    var totalImages = $('img').length;
                    var loadedImages = 0;

                    $('img').load(function() {
                        loadedImages++;
                        if (loadedImages === totalImages) {
                            self.scrollEnd();
                        }
                    });
                    setTimeout(function() {
                        self.$el.find('textarea').focus();

                    }, 200);



                    this.scrollEnd();
                    this.$el.find('div.container').append('<div class="blank"></div>');
                    this.normal();
                    setTimeout(function() {
                        self.normal();
                    }, 100);
                    if (device.platform === 'Android') {
                        setTimeout(function() {

                            $('.container').css({
                                'z-index': 2
                            }).css({
                                'z-index': 0
                            });
                        }, 1000);
                    }

                },
                getScrollHeight: function($el) {
                    if ($el.find('.chat_list') && this.$el.find('.chat_list').length > 0) {

                        return $el.find('.chat_list')[0].scrollHeight;
                    } else {
                        return 10000;
                    }
                },
                scrollEnd: function() {
                    var height = this.getScrollHeight(this.$el);
                    $('.chat_list').scrollTop(height);

                },
                scroll: function() {
                    // window.scrollTo(0, document.body.scrollHeight);
                    this.scrollEnd();
                },
                absolute: function() {
                    if (device.platform === '-..iOS') {

                        this.$('nav').css('position', 'absolute');
                        this.$('.blank').css('display', 'none');
                        this.$('.write').css('position', 'relative');
                        // setTimeout(function() {
                        //     this.$('nav').css('position', 'absolute');
                        // },100)
                    }
                },
                focused: function() {
                    this.scrollEnd();
                    //this.absolute();
                },
                sendImage: function() {

                    // //TODO: Pillar img de api...
                    // // var ruta = 'http://lorempixel.com/250/444/?date=' + moment.utc();
                    // var ruta = img.route_min;

                    // var msg = '<img class="js-open-image" src="' + ruta + '"  data-src="' + img.route + '" />';
                    // var mess = {
                    //     message: msg,
                    //     date: moment().unix(),
                    //     mine: true
                    // };
                    // this.trigger('sendmsg', mess);

                },
                sendmsg: function(e) {
                    e.preventDefault();

                    this.sendClicked = true;
                    var self = this;
                    var android = function() {
                        self.$('.js-newmsg').val('');
                        $('.js-newmsg').focus();
                        if (typeof cordova !== 'undefined') {

                            cordova.plugins.Keyboard.show();
                        }
                    };
                    var ios = function() {
                        $('.js-newmsg').focus();
                    };


                    var msg = this.$el.find('.js-newmsg').val();
                    msg = msg.replace(/\r?\n/g, '<br/>');

                    if (msg !== '') {
                        var mess = {
                            message: msg,
                            date: moment().unix(),
                            mine: true
                        };
                        this.$el.find('.js-newmsg').val('');
                        this.newMessage(mess);

                        this.scrollEnd();
                        if (device.platform === 'iOS') {

                            ios();
                        } else {
                            android();
                        }
                        this.trigger('sendmsg', mess);
                    }
                },
                //Checks if there are more than 2 lines in tecxarea. in this case, while textarea is focussed it shows 4 lines instead 2
                checkTextAreaSize: function() {
                    var $elem = this.$el.find('.js-newmsg');
                    setTimeout(function() {

                        if ($elem.height() < 50 && $elem[0].scrollHeight > 50) {
                            this.$('.write').addClass('multiline');
                        }
                    }, 1);
                }

            });
        });

        return App.chat.conversation.views;
    });
