

define(['App',
    'hbs!apps/login/main/templates/login',
    'funcs',
    'jquery',
    'toggles',
    'hbs!apps/login/main/templates/info_30_days',
    'hbs!apps/login/main/templates/info_message',
    'hbs!apps/login/main/templates/closeapp',
    'hbs!apps/login/main/templates/alert',
    'hbs!apps/login/main/templates/pendValidation',
    'hbs!apps/login/main/templates/datepicker',
    'i18next'


], function(App, mainTpl, funcs, $, togg, Info30DaysTpl, InfoMessageTpl, closeTPL, passAlert, pendValidationTpl, DatePickerTpl, i18n) {
    App.module('login.main.views', function(Module, App, Backbone) {

        //var main = Handlebars.compile(mainTpl);

        Module.Login = Backbone.Marionette.ItemView.extend({
            template: mainTpl,
            triggers: {
                'click a.forgotten': 'forgotten'
            },
            events: {
                'click .register': 'register',
                'submit': 'remember'
            },

            onShow: function() {

                var self = this;

                this.$el.trigger('click');
                setTimeout(function() {

                    this.$('input').blur();
                }, 200);
            },
            register: function(e) {

                e.stopPropagation();
                e.preventDefault();
                this.trigger('register');

            },
            stop: function() {},
            remember: function(e) {
                e.preventDefault();
                var email = this.$el.find('input[name="email"]').val();
                var passwd = this.$el.find('input[name="passwd"]').val();


                if ($('input[name="rememberme"]').prop('checked')) {

                    this.trigger('remember', email, passwd);
                } else {
                    this.trigger('submit', email, passwd);
                }

            },
            clearError: function() {

                funcs.clearError();
            },
            setPassError: function(option) {
                if (option === 0) {
                    funcs.addError('input[name="passwd"]', i18n.t('2_register.mandatory'));
                }
                if (option === 1) {
                    funcs.addError('input[name="passwd"]', i18n.t('2_register.min_6_chars'));
                }


            }
        });
        Module.DatePicker = Backbone.Marionette.ItemView.extend({
            template: DatePickerTpl,
            events: {
                'click button.submit': 'validate',
                'focus input,select': 'clearError',
                'click div[data-name="edad_she_sel"]': 'changeShe',
                'click div[data-name="edad_he_sel"]': 'changeHe'

            },


            changeDate: function(name) {

                var self = this;
                var options = {
                    date: new Date(),
                    mode: 'date'
                };

                datePicker.show(options, function(date) {

                    if (moment(date).isValid()) {
                        var val = moment(date).format('MM/DD/YYYY');
                        self.$el.find('div[data-name="' + name + '"]').text(val);

                    }

                });
            },
            changeShe: function(e) {
                this.changeDate('edad_she_sel');
                e.preventDefault();


            },
            changeHe: function(e) {
                this.changeDate('edad_he_sel');
                e.preventDefault();
            },
            onRender: function() {
                this.shePlaceholder = this.$el.find('div[data-name="edad_she_sel"]').text();
                this.hePlaceholder = this.$el.find('div[data-name="edad_he_sel"]').text();
            },

            validate: function(e, forcevalidate) {
                if (e) {
                    e.preventDefault();
                }
                var esMenor = function(date) {
                    var age = moment(date);

                    return moment().diff(age, 'years') < 18;
                };
                funcs.clearError();
                var errors = false;

                var she = this.$el.find('div[data-name="edad_she_sel"]').text();
                var he = this.$el.find('div[data-name="edad_he_sel"]').text();

                if (she === this.shePlaceholder && he === this.hePlaceholder) {
                    errors = true;
                    funcs.addError('div[data-name="edad_she_sel"]', i18n.t('2_register.mandatory')); //TODO: Canviar error
                }

                if (she !== this.shePlaceholder && esMenor(she)) {
                    errors = true;
                    funcs.addError('div[data-name="edad_she_sel"]', i18n.t('2_register.adult'));
                }
                if (he !== this.hePlaceholder && esMenor(he)) {
                    errors = true;
                    funcs.addError('div[data-name="edad_he_sel"]', i18n.t('2_register.adult'));
                }

                if ((!errors && e) || (!errors && forcevalidate)) {

                    this.trigger('changeDate', {
                        he: he,
                        she: she
                    });
                }

            },
            clearError: function() {
                funcs.clearError();
            }
        });
        Module.Info30Days = Backbone.Marionette.ItemView.extend({
            template: Info30DaysTpl,
            triggers: {

                'click .back': 'back',
                'click .continue': 'back',
                'click .upload_pict': 'upload_pict'

            }
        });
        Module.InfoMessage = Backbone.Marionette.ItemView.extend({
            template: InfoMessageTpl,
            triggers: {

                'click .back': 'back',
                'click .js-accept': 'accept',
                'click .upload_pict': 'upload_pict'

            }
        });
        Module.CloseApp = Backbone.Marionette.ItemView.extend({
            template: closeTPL,
            triggers: {
                'click .upload_pict': 'upload_pict',
                'click .continue': 'next'
            }
        });
        Module.pendValidation = Backbone.Marionette.ItemView.extend({
            template: pendValidationTpl,
            triggers: {
                'click .close': 'close'
            }
        });
        Module.RememberPass = Backbone.Marionette.ItemView.extend({
            template: passAlert,
            triggers: {
                'click .back': 'back'
            },
            events: {
                'click .submit': 'remember',
                'focus input': 'clearError'
            },
            remember: function(e) {
                e.preventDefault();
                var email = this.$('input[name="email"]').val();
                this.trigger('remember', email);
            },
            setEmailError: function() {

                funcs.addError('input[name="email"]', i18n.t('2_register.mailerror'));

            },
            setInvalidEmailError: function() {
                funcs.addError('input[name="email"]', i18n.t('2_register.checkemailerror'));
            },
            clearError: function() {
                funcs.clearError();
            }
        });


    });



    return App.login.main.views;

});
