


define(['App',
        'underscore',
        'handlebars',
        'text!apps/map/templates/map.html',
        'hbs!apps/map/templates/layout',
        'hbs!apps/map/templates/menu',
        'hbs!apps/map/templates/list',
        'hbs!apps/map/templates/list_elem',
        'apps/common/lib/spinner',
        'funcs',
        'apps/common/lib/map_style'

    ],
    function(App, _, Handlebars, mapTpl, layoutTpl, menuTpl, listTpl, listElemTpl, spinner, funcs, mapStyle) {
        mapTpl = Handlebars.compile(mapTpl);


        App.module('map.views', function(Module, App, Backbone, Marionette, $) {
            var userInfo, search, profile;
            Module.addInitializer(function() {
                userInfo = App.request('entities:user_info');
                search = App.request('entities:search');
                profile = App.request('entities:profile');
            });
            Module.Map = Marionette.CompositeView.extend({

                itemView: Module.Point,


                hideMap: function() {
                    this.$el.hide();
                },
                showMap: function() {
                    this.$el.fadeIn();
                },

                onRender: function() {
                    $('#site-menu').show();
                    $('input, textarea').blur();

                },
                onShow: function() {

                    this.trigger('loaded_end');
                },
                changeVisible: function() {
                    if (userInfo.get('visibility')) {
                        $('.visible').show();

                    } else {
                        $('.visible').hide();
                    }
                    if (userInfo.get('inTwilight')) {
                        $('.mode').addClass('tw_active');
                    } else {
                        $('.mode').removeClass('tw_active');

                    }

                },
                hack: function() {
                    if (this.markerClustererClients) {

                        this.markerClustererClients.repaint();
                        this.markerClustererLocals.repaint();
                        this.markerClustererWomen.repaint();
                        this.markerClustererMen.repaint();
                        this.markerClustererParties.repaint();
                        this.myMarkerC.repaint();
                    }
                    if (this.googlemapsLoaded) {

                        google.maps.event.trigger(this.map, 'resize');
                        this.map.setZoom(this.map.getZoom());
                        //this.map.setCenter(this.lastCenter);

                    }
                },
                setZoom: function(number) {
                    this.map.setZoom(number);
                },
                //Cuando nos llama el navigator ya tenemos posicion...
                loadMap: function() {

                    var self = this;

                    setInterval(function() {
                        // body...

                        if (App.mainRegion.currentView && App.mainRegion.currentView.oid === 'map_view') {

                            //google.maps.event.trigger(this.map, 'resize');
                        }
                    }, 1000);


                    var mapOptions = {
                        zoom: 11,
                        maxZoom: 20,
                        minZoom: 5,
                        disableDefaultUI: true,
                        center: new google.maps.LatLng(parseFloat(userInfo.get('lat')), parseFloat(userInfo.get('lon'))),
                        styles: mapStyle
                    };

                    this.lastCenter = new google.maps.LatLng(parseFloat(userInfo.get('lat')), parseFloat(userInfo.get('lon')));
                    self.map = new google.maps.Map(document.getElementById('map'), mapOptions);


                    google.maps.event.addListenerOnce(self.map, 'tilesloaded', function() {
                        //this part runs when the mapobject is created and rendered
                        google.maps.event.addListenerOnce(self.map, 'tilesloaded', function() {
                            self.googlemapsLoaded = true;
                        });
                    });

                    funcs.gmapHomeControl(self.map, google.maps.ControlPosition.RIGHT_BOTTOM, function() {

                        self.trigger('home');
                    });
                    funcs.gmapPlusMinus(self.map, google.maps.ControlPosition.LEFT_TOP);
                    funcs.gmapVisibility(self.map, google.maps.ControlPosition.LEFT_BOTTOM);
                    funcs.gmapLoading(self.map, google.maps.ControlPosition.RIGHT_TOP);
                    funcs.gmapWall(self.map, google.maps.ControlPosition.RIGHT_CENTER);
                    google.maps.event.addListener(self.map, 'idle', function() {
                        spinner(false);

                    });
                    google.maps.event.addListener(self.map, 'drag', function() {
                        if (App.mainRegion.currentView && App.mainRegion.currentView.oid === 'map_view') {
                            self.trigger('map_moved');
                            self.lastCenter = self.map.getCenter();
                        }
                    });
                    google.maps.event.addListener(self.map, 'dragend', function() {

                        if (App.mainRegion.currentView && App.mainRegion.currentView.oid === 'map_view') {
                            self.trigger('map_moved');
                            self.lastCenter = self.map.getCenter();
                            self.checkNewCenter();
                        }

                    });
                    //inicializamos
                    setTimeout(function() {
                        // body...
                        self.changeVisible();
                    }, 1000);



                    var info = {
                        lat: userInfo.get('lat'),
                        lon: userInfo.get('lon')
                    };
                    self.center = new google.maps.LatLng(info.lat, info.lon);
                    self.map.setCenter(self.center);

                },
                loadMyPoint: function() {

                    var point = this.myPos;
                    var self = this;
                    if (self.myMarkerC) {
                        self.myMarkerC.clearMarkers();
                    }
                    var img = 'assets/img/point_my.png';
                    // if (style) {
                    //     img = 'assets/img/point_tw.png';
                    // }
                    self.mylocation = new google.maps.Marker({
                        clickable: false,
                        icon: new google.maps.MarkerImage(img),
                        position: point

                    });
                    self.twlocation = new google.maps.Marker({
                        clickable: false,
                        icon: new google.maps.MarkerImage(img),
                        position: point

                    });

                    self.myMarkerC = new MarkerClusterer(self.map, [], {
                        maxZoom: 20,
                        gridSize: 1,

                        styles: [{
                            url: img,
                            height: 40,
                            width: 25,
                            anchorText: [-5, 0],
                            top: -4,
                            textSize: 1,
                            textColor: '#ffffff'
                        }],
                        clusterClass: 'cluster_center'
                    });

                    self.myMarkerC.addMarkers([self.mylocation, self.twlocation]);

                },
                setNewCenter: function(lat, lon) {

                    this.center = new google.maps.LatLng(lat, lon);
                    this.lastCenter = new google.maps.LatLng(lat, lon);
                    this.map.setCenter(this.center);
                },
                radius: 100,
                checkNewCenter: function() {

                    var circle = new google.maps.Circle({
                        center: this.map.getCenter(),
                        radius: this.radius * 1000
                    });
                    if (circle.getBounds().contains(this.center)) {

                    } else {
                        //this.refCenter = this.map.getCenter();
                        this.center = this.map.getCenter();
                        this.trigger('map_out');

                    }
                },
                updateMyPos: function(lat, lon) {
                    this.lastCenter = new google.maps.LatLng(lat, lon);
                    this.myPos = new google.maps.LatLng(lat, lon);

                    this.loadMyPoint();


                    if (userInfo.get('inTwilight')) {
                        $('.mode').addClass('tw_active');
                    } else {
                        $('.mode').removeClass('tw_active');

                    }


                },
                clearMarkers: function() {
                    if (this.map && this.markerClustererClients) {
                        this.markerClustererClients.clearMarkers();
                        this.markerClustererLocals.clearMarkers();
                        this.markerClustererWomen.clearMarkers();
                        this.markerClustererMen.clearMarkers();
                        this.markerClustererParties.clearMarkers();
                    }
                },
                loadPoints: function(points, cb) {

                    var self = this;

                    //Miramos si està iniciado el mapa.. sino.. no podemos hacer nada
                    if (this.map) {

                        var markersClients = [];
                        var markersLocals = [];
                        var markersWomen = [];
                        var markersMen = [];
                        var markersParties = [];


                        this.clusterCb = cb;

                        if (self.markerClustererClients) {
                            self.markerClustererClients.clearMarkers();
                            self.markerClustererLocals.clearMarkers();
                            self.markerClustererWomen.clearMarkers();
                            self.markerClustererMen.clearMarkers();
                            self.markerClustererParties.clearMarkers();
                            //self.markerClusterer.remove();
                        } else {
                            self.markerClustererClients = new MarkerClusterer(self.map, markersClients, {
                                maxZoom: 13,
                                gridSize: 30,
                                minimumClusterSize: 5,
                                styles: [{
                                    url: 'assets/img/point_sum.png',
                                    height: 40,
                                    width: 25,
                                    anchorText: [-5, 0],
                                    top: -4,
                                    textSize: 16,
                                    textColor: '#ffffff'

                                }]
                            });
                            self.markerClustererLocals = new MarkerClusterer(self.map, markersLocals, {
                                maxZoom: 13,
                                gridSize: 30,
                                minimumClusterSize: 5,
                                styles: [{
                                    url: 'assets/img/point_sum_1.png',
                                    height: 40,
                                    width: 25,
                                    anchorText: [-5, 0],
                                    textSize: 17,
                                    textColor: '#ffffff'
                                }]
                            });
                            self.markerClustererWomen = new MarkerClusterer(self.map, markersWomen, {
                                maxZoom: 13,
                                gridSize: 30,
                                minimumClusterSize: 5,
                                styles: [{
                                    url: 'assets/img/point_woman_group.png',
                                    height: 40,
                                    width: 25,
                                    anchorText: [-5, 0],
                                    top: -4,
                                    textSize: 16,
                                    textColor: '#ffffff'

                                }]
                            });
                            self.markerClustererMen = new MarkerClusterer(self.map, markersMen, {
                                maxZoom: 13,
                                gridSize: 30,
                                minimumClusterSize: 5,
                                styles: [{
                                    url: 'assets/img/point_man_group.png',
                                    height: 40,
                                    width: 25,
                                    anchorText: [-5, 0],
                                    top: -4,
                                    textSize: 16,
                                    textColor: '#ffffff'

                                }]
                            });
                            self.markerClustererParties = new MarkerClusterer(self.map, markersParties, {
                                maxZoom: 13,
                                gridSize: 30,
                                minimumClusterSize: 5,
                                styles: [{
                                    url: 'assets/img/point_party_group.png',
                                    height: 40,
                                    width: 25,
                                    anchorText: [-5, 0],
                                    textSize: 17,
                                    textColor: '#ffffff'
                                }]
                            });

                            google.maps.event.addListener(self.markerClustererClients, 'clusteringend', function() {
                                if (self.clusterCb) {
                                    self.clusterCb();
                                    self.clusterCb = undefined;
                                }
                            });



                        }

                        var profId = profile.get('id_client');
                        _.each(points, function(element) {

                            var icon, marker;

                            if (element.idl) {

                                //LOCAL
                                icon = 'assets/img/point_1.png';
                                marker = new google.maps.Marker({
                                    position: new google.maps.LatLng(parseFloat(element.la), parseFloat(element.lo)),
                                    icon: icon
                                });
                                google.maps.event.addListener(marker, 'click', function() {
                                    self.trigger('club_clicked', element);
                                });
                                markersLocals.push(marker);
                                //self.locals.push(marker);
                            } else if(element.idp){
                                icon = 'assets/img/point_party.png';
                                marker = new google.maps.Marker({
                                    position: new google.maps.LatLng(parseFloat(element.la), parseFloat(element.lo)),
                                    icon: icon,
                                    optimized: false,
                                    zIndex:99999999
                                });
                                google.maps.event.addListener(marker, 'click', function() {
                                    self.trigger('party_clicked', element);
                                });
                                markersParties.push(marker);

                            } else if (element.idc !== profId) {


                                //SWINGER
                                if ( parseInt(element.user_type,10) === 0 ) {


                                    if (parseInt(element.f, 10) === 1) {
                                        icon = 'assets/img/point_2.png';
                                    } else {
                                        icon = 'assets/img/point_5.png';
                                    }
                                    marker = new google.maps.Marker({
                                        position: new google.maps.LatLng(parseFloat(element.la), parseFloat(element.lo)),
                                        //map: self.map,
                                        icon: icon
                                    });
                                    google.maps.event.addListener(marker, 'click', function() {
                                        self.trigger('person_clicked', element);
                                    });
                                    markersClients.push(marker);

                                } else if (parseInt(element.user_type,10) === 1) {
                                    icon = 'assets/img/point_woman.png';
                                    marker = new google.maps.Marker({
                                        position: new google.maps.LatLng(parseFloat(element.la), parseFloat(element.lo)),
                                        //map: self.map,
                                        icon: icon
                                    });
                                    google.maps.event.addListener(marker, 'click', function() {
                                        self.trigger('person_clicked', element);
                                    });
                                    markersWomen.push(marker);

                                } else if (parseInt(element.user_type,10) === 2) {
                                    icon = 'assets/img/point_man.png';
                                    marker = new google.maps.Marker({
                                        position: new google.maps.LatLng(parseFloat(element.la), parseFloat(element.lo)),
                                        //map: self.map,
                                        icon: icon
                                    });
                                    google.maps.event.addListener(marker, 'click', function() {
                                        self.trigger('person_clicked', element);
                                    });
                                    markersMen.push(marker);

                                }
                            }
                        });

                        self.markerClustererClients.addMarkers(markersClients);
                        self.markerClustererLocals.addMarkers(markersLocals);
                        self.markerClustererWomen.addMarkers(markersWomen);
                        self.markerClustererMen.addMarkers(markersMen);
                        self.markerClustererParties.addMarkers(markersParties);
                        self.loadMyPoint();
                        //AKI EL ADD
                    } else {
                        setTimeout(function() {
                            self.loadPoints(points, cb);

                        }, 1000);
                    }
                },
                template: mapTpl

            });
        });


        return App.map.views;
    });
