
define(['App',
    'underscore',
    'hbs!apps/twilight/map/templates/map',
    'funcs',
    'apps/common/lib/map_style'
], function(App, _, mapTpl, funcs, mapStyle) {
    App.module('twilight.map.views', function(Module, App, Backbone, Marionette) {
        Module.Map = Marionette.ItemView.extend({
            template: mapTpl,

            /*   tagName: 'li',
                className: 'chat_elem',*/
            triggers: {
                'click .back': 'back',
                'submit form':'none'
            },
            onShow: function() {
                var self = this;
                var mapOptions = {
                    center: new google.maps.LatLng(this.model.get('lat'), this.model.get('lon')),
                    zoom: 13,
                    //minZoom: 10,
                    disableDefaultUI: true,
                    styles: mapStyle
                };
                this.map = new google.maps.Map(document.getElementById('tw-map'), mapOptions);

                this.circle = new google.maps.Circle({
                    center: new google.maps.LatLng(parseFloat(this.model.get('lat')), parseFloat(this.model.get('lon'))),
                    radius: 1000,
                    strokeColor: '#000000',
                    strokeOpacity: 0,
                    strokeWeight: 1,
                    fillColor: '#e74c3c',
                    fillOpacity: 0.2,
                    map: this.map
                });
                this.geocoder = new google.maps.Geocoder();
                this.geocoder.geocode({
                    location: new google.maps.LatLng(parseFloat(this.model.get('lat')), parseFloat(this.model.get('lon')))
                }, function(results, status) {
                    if (status === google.maps.GeocoderStatus.OK) {
                        self.name = results[0].formatted_address;
                    }
                });
                funcs.gmapHomeControl(this.map, google.maps.ControlPosition.RIGHT_BOTTOM);
            },
            initialize: function() {
                this.model.bind('all', this.render, this);
            },
            events: {
                'click .adv-search': 'search',
                'click li': 'setMarker',
                'click .js-submit': 'savePos',
                'click .input-adress': 'select',
                'keypress input':'activeSearch'
            },
            search: function() {
                var address = this.$('.input-adress').val();
                this.trigger('search', address);
            },
            activeSearch: function() {
                this.$('.adv-search').removeClass('busquedainactiva');
                this.$('.adv-search').addClass('busquedaactiva');
            },
            inactiveSearch: function() {
                this.$('.adv-search').removeClass('busquedaactiva');
                this.$('.adv-search').addClass('busquedainactiva');
            },
            select: function(e) {
                e.preventDefault();
                $(e.target).select();
                //this.activeSearch();
            },
            setOptions: function(elems) {
                var self = this;
                this.clear();
                _.each(elems, function(el) {
                    var dom = $('<li/>').text(el.formatted_address);
                    dom.click(function() {
                        self.setMarker(el.geometry.location, el.formatted_address);
                        self.name = el.formatted_address;
                        self.$('input[name=adress]').val(el.formatted_address);
                    });
                    self.$('ul').append(dom);
                });
            },
            setMarker: function(obj) {
                var latlan = obj;
                // var latlan = new google.maps.LatLng(parseFloat(parseFloat(obj.k)), parseFloat(parseFloat(obj.B)));
                //this.marker.setPosition(latlan);
                this.circle.setCenter(latlan);
                this.map.setCenter(latlan);
                this.clear();
                this.showSave();
            },
            showSave: function() {
                this.$('.js-submit').fadeIn();
            },
            clear: function() {
                this.$('ul').html('');
            },
            savePos: function() {
                this.trigger('savePos', this.circle.getCenter(), this.name);
            }
        });
    });
    return App.twilight.map.views;
});
